
/*位置推算に地球位置関数earthposition()を参照。

    CK12S010  2013 11 28.7672  0.012487  1.000005  345.5222  295.7166   61.9802  20130304   7.0  3.2  C/2012 S1 (ISON)                                         MPEC 2013-E06

軌道タイプ
近日点通過（1970年からのミリ秒）
近日点距離
離心率
近日点引数
昇交点黄経
軌道傾斜角
元期
光度係数
標準等級
名称
出典

var packedname = str.slice(0,14);
var T = str.slice(15,31);
var q = str.slice(31,40);
var e = str.slice(41,50);
var ω = str.slice(51,60);
var Ω = str.slice(61,70);
var i = str.slice(71,80);
var E = str.slice(82,89);
var k = str.slice(92,96);
var H = str.slice(98.102);
var name = str.slice(103,158);
var MPEC = str.slice(160);

*/

//MPCフォーマットから軌道要素を抜き出す。
function readMPCformat(str){
var oe = new Array();

oe[0] = str.slice(0,14);
oe[1] = str.slice(14,30);
oe[2] = str.slice(30,40);
oe[3] = str.slice(41,50);
oe[4] = str.slice(51,60);
oe[5] = str.slice(61,70);
oe[6] = str.slice(71,80);
oe[7] = str.slice(81,89);
oe[8] = str.slice(90,96);
oe[9] = str.slice(96,102);
oe[10] = str.slice(102,158);
oe[11] = str.slice(159,172);

return oe;

}

//yyyy mm dd.dddd形式の日付を1970年ミリ秒数に変換。dは小数点以下桁数
function TtoUNIXTime(num,d){
var year = num.slice(0,4);
var month = num.slice(5,7);
var fullday = num.slice(8,d+11);//小数の日
var day = Math.floor(fullday);//整数の日
var dday = fullday - day;//日の小数部分
var hours = Math.floor(24*dday);//時
var min = Math.floor(60*(24*dday - hours));//分
var sec = Math.floor(60*(60*(24*dday - hours)- min));
var millisec = Math.floor((60*(60*(24*dday - hours)- min) - sec)*1000);
var sT = new Date(year,month - 1,day,hours,min,sec,millisec);

return sT.getTime();
}



//軌道面上の天体位置
function orbitXY(num,T,q,e){//ミリ秒の時刻、近日点時刻、距離、離心率
var GS = 0.000295912208;//日心重力定数GS　au^3/day^2

if(e != 1){//放物線以外の時、軌道半長径を求める
var a = Math.abs(q / (e - 1));//軌道半長径
}

//平均運動
if(e == 1){//放物線軌道
var n = Math.sqrt(GS / (2 * Math.pow(q,3)));//平均運動
}else if(e != 1){//楕円軌道の時
var n = Math.sqrt(GS / Math.pow(a,3));//平均運動
}

//平均近点角
var l = n * (num - T)/86400000;//単位のミリ秒を日単位に

//■放物線軌道
if(e == 1){
var G = Math.pow(Math.tan(0.5 * Math.atan2(2,3*l)),1/3)
var f = 2 * Math.atan2((1-G*G),G);//真近点角f
var ff = 1 + Math.cos(f);
var rr = 2*q / ff;//動径（日心距離）
var xx = 2*q*Math.cos(f) / ff;//軌道面の座標値X
var yy = 2*q*Math.sin(f) / ff;//軌道面の座標値Y

//■楕円軌道
}else if(e < 1){
var u = new Array();//離心近点角
//u[0] = l/(1 - e);//離心近点角の近似値
u[0] = l+(e - e*e*e/8)*Math.sin(l) + ((1/2)*e*e)*Math.sin(2*l);//離心近点角の近似値

var Du = (l + e*Math.sin(u[0]) - u[0])/(1 - e*Math.cos(u[0]));//補正値の初期値

//繰り返し近似（13回）
for(var j=0;Math.abs(Du) > 0.000000000001;j++){//iが使われてるので仕方なくj
if(j == 100) break;//10000回やってダメなら諦める
Du = (l + e*Math.sin(u[j]) - u[j])/(1 - e*Math.cos(u[j]));
u[j+1] = u[j]+Du;//離心近点角
}
var rr = a*(1 - e*Math.cos(u[j]));//動径（日心距離）
var xx = a*(Math.cos(u[j]) - e);//軌道面の座標値X
var yy = a*Math.sqrt(1 - e*e)*Math.sin(u[j]);//軌道面の座標値Y

//■双曲線軌道
}else if(e > 1){
var u = new Array();//離心近点角
u[0] = l/(e - 1);//離心近点角の近似値
var Du = (l - e*sinh(u[0]) + u[0])/(e*cosh(u[0]) - 1);//補正値の初期値

//繰り返し近似（13回）
for(var j=0;Math.abs(Du) > 0.000000000001;j++){
if(j == 100) break;//100回やってダメなら諦める
Du = (l - e*sinh(u[j]) + u[j])/(e*cosh(u[j]) - 1);
u[j+1] = u[j]+Du;//離心近点角
}
var rr = a*(e*cosh(u[j]) - 1);//動径（日心距離）
var xx = a*(e - cosh(u[j]));//軌道面の座標値X
var yy = a*Math.sqrt(e*e - 1)*sinh(u[j]);//軌道面の座標値Y
}

var AA = new Array();
AA[0] = rr;
AA[1] = xx;
AA[2] = yy;
if(e != 1){
AA[3] = Du;
}

return AA;
}//function orbitXY



function orbit(type,num,T,q,e,pdeg,Ndeg,ideg,E,k,H,name,note){
var rad = Math.PI/180;

if(type == "d"){//彗星軌道 軌道要素は度単位 時刻は1970年からのミリ秒単位
//地球位置
var Earth = new Array();

Earth = earthposition(num);
var ERR = Earth[3];//地球の地心距離
var Exx = Earth[12];//地球の赤道直交座標2000.0
var Eyy = Earth[13];
var Ezz = Earth[14];
var Exxn = Earth[15];//地球の赤道直交座標瞬時
var Eyyn = Earth[16];
var Ezzn = Earth[17];
var αSun2000 = Earth[31]/15+12;//赤経。時単位
var δSun2000 = -Earth[32];//赤緯。度単位

//ラジアンに変更

//var T = Tto1970Millisec(t,4);

var p = pdeg*Math.PI/180;
var N = Ndeg*Math.PI/180;
var i = ideg*Math.PI/180;

/*
var GS = 0.000295912208;//日心重力定数GS　au^3/day^2

if(e != 1){//放物線以外の時、軌道半長径を求める
var a = Math.abs(q / (e - 1));//軌道半長径
}

//平均運動
if(e == 1){//放物線軌道
var n = Math.sqrt(GS / (2 * Math.pow(q,3)));//平均運動
}else if(e != 1){//楕円軌道の時
var n = Math.sqrt(GS / Math.pow(a,3));//平均運動
}



//平均近点角
var l = n * (num - T);

//■放物線軌道
if(e == 1){
var G = Math.pow(Math.tan(0.5 * Math.atan2(2,3*l)),1/3)
var f = 2 * Math.atan2((1-G*G),G);//真近点角f
var ff = 1 + Math.cos(f);
var rr = 2*q / ff;//動径（日心距離）
var xx = 2*q*Math.cos(f) / ff;//軌道面の座標値X
var yy = 2*q*Math.sin(f) / ff;//軌道面の座標値Y

//■楕円軌道
}else if(e < 1){
var u = new Array();//離心近点角
u[0] = l/(1 - e);//離心近点角の近似値
var Du = (l + e*Math.sin(u[0]) - u[0])/(1 - e*Math.cos(u[0]));//補正値の初期値

//繰り返し近似（13回）
for(var j=0;Math.abs(Du) > 0.000000000001;j++){//iが使われてるので仕方なくj
if(j == 1000) break;//10000回やってダメなら諦める
Du = (l + e*Math.sin(u[j]) - u[j])/(1 - e*Math.cos(u[j]));
u[j+1] = u[j]+Du;//離心近点角
}
var rr = a*(1 - e*Math.cos(u[j]));//動径（日心距離）
var xx = a*(Math.cos(u[j]) - e);//軌道面の座標値X
var yy = a*Math.sqrt(1 - e*e)*Math.sin(u[j]);//軌道面の座標値Y

//■双曲線軌道
}else if(e > 1){
var u = new Array();//離心近点角
u[0] = l/(e - 1);//離心近点角の近似値
var Du = (l - e*sinh(u[0]) + u[0])/(e*cosh(u[0]) - 1);//補正値の初期値

//繰り返し近似（13回）
for(var j=0;Math.abs(Du) > 0.000000000001;j++){
if(j == 100) break;//100回やってダメなら諦める
Du = (l - e*sinh(u[j]) + u[j])/(e*cosh(u[j]) - 1);
u[j+1] = u[j]+Du;//離心近点角
}
var rr = a*(e*cosh(u[j]) - 1);//動径（日心距離）
var xx = a*(e - cosh(u[j]));//軌道面の座標値X
var yy = a*Math.sqrt(e*e - 1)*sinh(u[j]);//軌道面の座標値Y

}
*/
var AA0 = new Array();
AA0 = orbitXY(num,T,q,e);
var rr = AA0[0];//日心距離
var xx = AA0[1];//軌道面の座標値X
var yy = AA0[2];//軌道面の座標値Y

var AA = new Array();
AA[0] = rr;//日心距離
AA[1] = xx;//軌道面の座標値X
AA[2] = yy;//軌道面の座標値Y
//AA[3] = AA0[3];
var Xc = xx*(Math.cos(N)*Math.cos(p) - Math.sin(N)*Math.cos(i)*Math.sin(p)) - yy*(Math.cos(N)*Math.sin(p) + Math.sin(N)*Math.cos(i)*Math.cos(p));
var Yc = xx*(Math.sin(N)*Math.cos(p) + Math.cos(N)*Math.cos(i)*Math.sin(p)) - yy*(Math.sin(N)*Math.sin(p) - Math.cos(N)*Math.cos(i)*Math.cos(p));
var Zc = xx* Math.sin(i)*Math.sin(p) + yy*Math.sin(i)*Math.cos(p);

AA[6] = Xc;//黄道直交座標X（2000.0）
AA[7] = Yc;//黄道直交座標Y
AA[8] = Zc;//黄道直交座標Z

var ε = 23.43929111*Math.PI/180;//2000.0の黄道傾斜角ε




//var eee = 23.445787*Math.PI/180;//1950年

var qq = new Array();
qq =changeR(1,Xc,Yc,Zc,ε);
AA[12] = qq[0];//赤道直交座標X（2000.0）
AA[13] = qq[1];//赤道直交座標Y
AA[14] = qq[2];//赤道直交座標Z

var qqn = new Array();
qqn =changeR(2,qq[0],qq[1],qq[2],num);
AA[15] = qqn[0];//赤道直交座標X（瞬時）
AA[16] = qqn[1];//
AA[17] = qqn[2];//

//AA[10] = Du;//赤道直交座標Z


var Δ = Math.sqrt(Math.pow((AA[12] - Exx),2) + Math.pow((AA[13] - Eyy),2) + Math.pow((AA[14] - Ezz),2));

var α = Math.atan2((AA[13] - Eyy),(AA[12] - Exx))*180/Math.PI;//2000.0の彗星の赤経。度
if(α < 0){α = α + 360;}

var δ = (Math.asin((AA[14] - Ezz)/Δ))*180/Math.PI;//2000.0の赤緯

var αn = Math.atan2((AA[16] - Eyyn),(AA[15] - Exxn))*180/Math.PI;//視赤経。
if(αn < 0){αn = αn + 360;}
var δn = Math.asin((AA[17] - Ezzn)/Δ)*180/Math.PI;//視赤緯。

AA[20] = Δ;//地心距離

var sc =  180/Math.PI*Math.acos((Δ*Δ + ERR*ERR - rr*rr)/(2*Δ*ERR));
AA[21] = sc;//太陽離角

var β = 180/Math.PI*Math.acos((rr*rr + Δ*Δ - ERR*ERR)/(2*Δ*rr));
AA[22] = β;//太陽離角

AA[31] = α;//2000.0の彗星の赤経
AA[32] = δ;//2000.0の彗星の赤緯

/*
var λp = N - 90;//軌道面の北極の黄経
var βp = 90 - i;//黄緯

var pl = new Array();
pl = changeAL(2,λp,βp,23.43929111);
var αp = pl[0];
var δp = pl[1];


var De = -Math.asin(Math.sin(δp*rad)*Math.sin(δ*rad) + Math.cos(δp*rad)*Math.cos(δ*rad)*Math.cos((αp - α)*rad));
AA[9] = De*180/Math.PI;//単位は度


*/
//太陽の方向角
var Δα = (αSun2000 - α/15) * 15 * Math.PI/180;//（2点の赤経差。ラジアン）
var VSun = 180/Math.PI * Math.atan2(Math.sin(Δα), Math.cos(δ * Math.PI/180) * Math.tan(δSun2000 * Math.PI/180) - Math.sin(δ * Math.PI/180) * Math.cos(Δα));
if(VSun <= 0){
VSun = VSun + 360;
}
AA[33] = VSun;//太陽の方向角

//尾の方向角
var VSunA;
VSunA = VSun + 180;
if (VSunA >= 360){
VSunA = VSunA - 360;
}
AA[34] = VSunA;//尾の方向角


AA[41] = αn;//彗星の視赤経
AA[42] = δn;//彗星の視赤緯
//AA[100] = Du;//彗星の視赤緯

//太陽位置
AA[131] = Earth[31];//2000.0の彗星の赤経
AA[132] = Earth[32];//2000.0の彗星の赤緯
AA[141] = Earth[19];//彗星の視赤経
AA[142] = Earth[20];//彗星の視赤緯

return AA;
}
}
